#!/usr/bin/env python3

import subprocess
import re
import os

iface_name = "eth0"

#
def parse_ifconfig(string):

  strings = string.split('\n')

  for i in range(len(strings)):
    tmpstr = strings[i]
    if tmpstr[0:len(iface_name)]==iface_name:
      tmpstr = strings[i+1]
      m1 = re.search('(?<=addr:)(\d+\.){3}\d+',tmpstr)
      ipaddr = m1.group(0)
      m1 = re.search('(?<=Mask:)(\d+\.){3}\d+',tmpstr)
      mask = m1.group(0)
      return ipaddr,mask

  return None,None

#
def parse_route(string):

  strings = string.strip().split('\n')

  for s in strings[2:]:
    ss = s.split()
    if ss[1]!="0.0.0.0":
      return ss[1]

  return None

#
def get_current_network_config():

  ifconfig_run_output = subprocess.run('ifconfig', stdout=subprocess.PIPE)
  ipaddr, mask = parse_ifconfig(ifconfig_run_output.stdout.decode())

  route_run_output = subprocess.run(['route','-n'], stdout=subprocess.PIPE)
  gateway = parse_route(route_run_output.stdout.decode())

  return ipaddr, mask, gateway


# MAIN
if __name__ == "__main__":

  # run from /home/root, checking
  # print(os.getcwd())

  print("Step 0: Start")

  # step 1: unpack, keep rights
  print("Step 1: Unpack")
  subprocess.run(['mkdir','-p','/tmp/rootfs'], stdout=subprocess.PIPE)
  subprocess.run(['tar','-C','/tmp/rootfs','-xzpf','/tmp/rootfs.tar.gz'], stdout=subprocess.PIPE)

  # step 2: get network settings
  print("Step 2: Read current network settings")
  ipaddr, mask, gateway = get_current_network_config()

  print("Settings for /etc/network/interfaces:")
  print("  ipaddr:  "+ipaddr)
  print("  netmask: "+mask)
  print("  gateway: "+gateway)

  # step 3: modify interfaces
  print("Step 3: Apply network settings to file")

  with open("/tmp/rootfs/etc/network/interfaces", "r") as f:
    old = f.read()

  # unique lines, this is luck
  new = re.sub('(?<=address\ )(\d+\.){3}\d+',ipaddr,old)
  new = re.sub('(?<=netmask\ )(\d+\.){3}\d+',mask,new)
  new = re.sub('(?<=gateway\ )(\d+\.){3}\d+',gateway,new)

  with open("/tmp/rootfs/etc/network/interfaces", "w") as f:
    f.write(new)

  # step 4: rsync with the rootfs
  print("Step 4: Overwrite filesystem")
  subprocess.run(['rsync','-a','/tmp/rootfs/','/'], stdout=subprocess.PIPE)

  print("Step 5: Request syncing (if boot from flash)")
  subprocess.run(['rsync','-a','/tmp/rootfs/','/'], stdout=subprocess.PIPE)

  subprocess.run(['sync'], stdout=subprocess.PIPE)
  subprocess.run(['overlay_sync','1'], stdout=subprocess.PIPE)

  print("""\033[92mNEXT:
    * if boot from SD card - simply reboot
    * if boot from flash -
      ~$ ssh root@{0}
      root@elphel393:~# shutdown -hP now
      then powercycle\033[0m
  """.format(ipaddr))

  print("Success")
